@@ -99,7 +99,9 @@ class Env:
 
     @lazy_property
     def conda(self):
-        return Conda(self._container_img)
+        return Conda(
+            container_img=self._container_img, frontend=self.frontend, check=True
+        )
 
     @lazy_property
     def pin_file(self):
@@ -626,7 +628,7 @@ class Conda:
     instances = dict()
     lock = threading.Lock()
 
-    def __new__(cls, container_img=None, prefix_path=None):
+    def __new__(cls, container_img=None, prefix_path=None, frontend=None, check=False):
         with cls.lock:
             if container_img not in cls.instances:
                 inst = super().__new__(cls)
@@ -635,7 +637,9 @@ class Conda:
             else:
                 return cls.instances[container_img]
 
-    def __init__(self, container_img=None, prefix_path=None):
+    def __init__(
+        self, container_img=None, prefix_path=None, frontend=None, check=False
+    ):
         if not self.is_initialized:  # avoid superfluous init calls
             from snakemake.deployment import singularity
             from snakemake.shell import shell
@@ -643,10 +647,12 @@ class Conda:
             if isinstance(container_img, singularity.Image):
                 container_img = container_img.path
             self.container_img = container_img
+            self.frontend = frontend
 
             self.info = json.loads(
                 shell.check_output(
-                    self._get_cmd("conda info --json"), universal_newlines=True
+                    self._get_cmd(f"conda info --json"),
+                    universal_newlines=True,
                 )
             )
 
@@ -658,7 +664,10 @@ class Conda:
             self.platform = self.info["platform"]
 
             # check conda installation
-            self._check()
+            if check:
+                if frontend is None:
+                    raise ValueError("Frontend must be specified if check is True.")
+                self._check()
 
     @property
     def is_initialized(self):
@@ -672,48 +681,64 @@ class Conda:
     def _check(self):
         from snakemake.shell import shell
 
-        # Use type here since conda now is a function.
-        # type allows to check for both functions and regular commands.
-        if not ON_WINDOWS or shell.get_executable():
-            locate_cmd = "type conda"
-        else:
-            locate_cmd = "where conda"
+        frontends = ["conda"]
+        if self.frontend == "mamba":
+            frontends = ["mamba", "conda"]
 
-        try:
-            shell.check_output(self._get_cmd(locate_cmd), stderr=subprocess.STDOUT)
-        except subprocess.CalledProcessError as e:
-            if self.container_img:
-                raise CreateCondaEnvironmentException(
-                    "The 'conda' command is not "
-                    "available inside "
-                    "your singularity container "
-                    "image. Snakemake mounts "
-                    "your conda installation "
-                    "into singularity. "
-                    "Sometimes, this can fail "
-                    "because of shell restrictions. "
-                    "It has been tested to work "
-                    "with docker://ubuntu, but "
-                    "it e.g. fails with "
-                    "docker://bash "
-                )
+        for frontend in frontends:
+            # Use type here since conda now is a function.
+            # type allows to check for both functions and regular commands.
+            if not ON_WINDOWS or shell.get_executable():
+                locate_cmd = f"type {frontend}"
             else:
-                raise CreateCondaEnvironmentException(
-                    "The 'conda' command is not "
-                    "available in the "
-                    "shell {} that will be "
-                    "used by Snakemake. You have "
-                    "to ensure that it is in your "
-                    "PATH, e.g., first activating "
-                    "the conda base environment "
-                    "with `conda activate base`.".format(shell.get_executable())
-                )
+                locate_cmd = f"where {frontend}"
+
+            try:
+                shell.check_output(self._get_cmd(locate_cmd), stderr=subprocess.STDOUT)
+            except subprocess.CalledProcessError as e:
+                if self.container_img:
+                    msg = (
+                        f"The '{frontend}' command is not "
+                        "available inside "
+                        "your singularity container "
+                        "image. Snakemake mounts "
+                        "your conda installation "
+                        "into singularity. "
+                        "Sometimes, this can fail "
+                        "because of shell restrictions. "
+                        "It has been tested to work "
+                        "with docker://ubuntu, but "
+                        "it e.g. fails with "
+                        "docker://bash "
+                    )
+                else:
+                    msg = (
+                        f"The '{frontend}' command is not "
+                        "available in the "
+                        f"shell {shell.get_executable()} that will be "
+                        "used by Snakemake. You have "
+                        "to ensure that it is in your "
+                        "PATH, e.g., first activating "
+                        "the conda base environment "
+                        "with `conda activate base`."
+                    )
+                if frontend == "mamba":
+                    msg += (
+                        "The mamba package manager (https://github.com/mamba-org/mamba) is a "
+                        "fast and robust conda replacement. "
+                        "It is the recommended way of using Snakemake's conda integration. "
+                        "It can be installed with `conda install -n base -c conda-forge mamba`. "
+                        "If you still prefer to use conda, you can enforce that by setting "
+                        "`--conda-frontend conda`.",
+                    )
+                raise CreateCondaEnvironmentException(msg)
+
         try:
             self._check_version()
             self._check_condarc()
         except subprocess.CalledProcessError as e:
             raise CreateCondaEnvironmentException(
-                "Unable to check conda installation:\n" + e.stderr.decode()
+                f"Unable to check conda installation:" "\n" + e.stderr.decode()
             )
 
     def _check_version(self):
