@@ -1,5 +1,5 @@
 __authors__ = "Johannes Köster"
-__copyright__ = "Copyright 2021, Johannes Köster"
+__copyright__ = "Copyright 2022, Johannes Köster"
 __email__ = "johannes.koester@uni-due.de"
 __license__ = "MIT"
 
@@ -26,6 +26,7 @@ class RuleInfo:
         self.shadow_depth = None
         self.resources = None
         self.priority = None
+        self.retries = None
         self.version = None
         self.log = None
         self.docstring = None
@@ -33,6 +34,7 @@ class RuleInfo:
         self.script = None
         self.notebook = None
         self.wrapper = None
+        self.template_engine = None
         self.cwl = None
         self.cache = False
         self.path_modifier = None
