@@ -1,5 +1,5 @@
 __author__ = "Johannes Köster"
-__copyright__ = "Copyright 2021, Johannes Köster"
+__copyright__ = "Copyright 2022, Johannes Köster"
 __email__ = "johannes.koester@uni-due.de"
 __license__ = "MIT"
 
@@ -114,6 +114,13 @@ class shell:
                 cls._processes[jobid].kill()
                 del cls._processes[jobid]
 
+    @classmethod
+    def terminate(cls, jobid):
+        with cls._lock:
+            if jobid in cls._processes:
+                cls._processes[jobid].terminate()
+                del cls._processes[jobid]
+
     @classmethod
     def cleanup(cls):
         with cls._lock:
@@ -169,13 +176,13 @@ class shell:
         if conda_env:
             if ON_WINDOWS and not cls.get_executable():
                 # If we use cmd.exe directly on winodws we need to prepend batch activation script.
-                cmd = Conda(container_img, prefix_path=conda_base_path).shellcmd_win(
-                    conda_env, cmd
-                )
+                cmd = Conda(
+                    container_img=container_img, prefix_path=conda_base_path
+                ).shellcmd_win(conda_env, cmd)
             else:
-                cmd = Conda(container_img, prefix_path=conda_base_path).shellcmd(
-                    conda_env, cmd
-                )
+                cmd = Conda(
+                    container_img=container_img, prefix_path=conda_base_path
+                ).shellcmd(conda_env, cmd)
 
         tmpdir = None
         if len(cmd.replace("'", r"'\''")) + 2 > MAX_ARG_LEN:
@@ -198,7 +205,9 @@ class shell:
             )
             logger.info("Activating singularity image {}".format(container_img))
         if conda_env:
-            logger.info("Activating conda environment: {}".format(conda_env))
+            logger.info(
+                "Activating conda environment: {}".format(os.path.relpath(conda_env))
+            )
 
         tmpdir_resource = resources.get("tmpdir", None)
         # environment variable lists for linear algebra libraries taken from:
